@@ -12,6 +12,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <pthread.h>
+#include <glib.h>
 
 #include "list.h"
 #include "common.h"
@@ -21,75 +22,97 @@
 #include "smb_conn_proto.h"
 #include "process.h"
 #include "smb_conn.h"
+#include "neg_cache.h"
 
 #ifndef MAP_ANONYMOUS
     #define	MAP_ANONYMOUS	MAP_ANON
 #endif
 
+#define SMB_CONN_PROCESS_STATE_UNKNOWN	-1
+#define SMB_CONN_PROCESS_STATE_ALIVE	0
+#define SMB_CONN_PROCESS_STATE_DIED	1
 
-int		smb_conn_max_retry_count	= 3;
-int		smb_conn_max_passwd_query_count	= 10;
-int		smb_conn_server_reply_timeout	= 60;
-pthread_mutex_t	m_smb_conn			= PTHREAD_MUTEX_INITIALIZER;
+struct smb_conn_query_result{
+    int	errno_value;
+    int process_state;
+};
+
+
+static int		smb_conn_max_retry_count	= 2;
+static int		smb_conn_max_passwd_query_count	= 10;
+static int		smb_conn_server_reply_timeout	= 60;
+
+static inline int smb_conn_is_neg_cache_candidate(enum smb_conn_cmd query_cmd,
+                                                  struct smb_conn_query_result *result){
+    if (result->process_state == SMB_CONN_PROCESS_STATE_DIED)
+	switch(result->errno_value){
+	    case ETIMEDOUT:
+	    case ECONNREFUSED:
+	    case EHOSTUNREACH:
+		return 1;
+	    case EINVAL:
+		switch(query_cmd){
+		    case OPENDIR:
+		    case STAT:
+			return 1;
+		    default:
+			break;
+		}
+	    default:
+		break;
+	}
+    return 0;
+}
+
+static inline int smb_conn_query_result_map(struct smb_conn_query_result *result){
+    if (result->process_state == SMB_CONN_PROCESS_STATE_ALIVE)
+	return result->errno_value;
+    return EIO;
+}
+
+static inline int smb_conn_query_result_check(enum smb_conn_cmd query_cmd,
+                                              struct smb_conn_query_result *result){
+    if (result->process_state == SMB_CONN_PROCESS_STATE_ALIVE)
+	return 1;
+    if (smb_conn_is_neg_cache_candidate(query_cmd, result))
+	return 1;
+    return 0;
+}
 
 int smb_conn_set_max_retry_count(int count){
-    if (count < 1) return 0;
+    if (count < 0) return 0;
     DPRINTF(7, "count=%d\n", count);
-    pthread_mutex_lock(&m_smb_conn);
-    smb_conn_max_retry_count = count;
-    pthread_mutex_unlock(&m_smb_conn);
+    g_atomic_int_set(&smb_conn_max_retry_count, count);
     return 1;
 }
 
-int smb_conn_get_max_retry_count(void){
-    int count;
-
-    pthread_mutex_lock(&m_smb_conn);
-    count = smb_conn_max_retry_count;
-    pthread_mutex_unlock(&m_smb_conn);
-    DPRINTF(7, "count=%d\n", count);
-    return count;
+static inline int smb_conn_get_max_retry_count(void){
+    return g_atomic_int_get(&smb_conn_max_retry_count);
 }
 
 int smb_conn_set_max_passwd_query_count(int count){
     if (count < 3) return 0;
     DPRINTF(7, "count=%d\n", count);
-    pthread_mutex_lock(&m_smb_conn);
-    smb_conn_max_passwd_query_count = count;
-    pthread_mutex_unlock(&m_smb_conn);
+    g_atomic_int_set(&smb_conn_max_passwd_query_count, count);
     return 1;
 }
 
-int smb_conn_get_max_passwd_query_count(void){
-    int count;
-
-    pthread_mutex_lock(&m_smb_conn);
-    count = smb_conn_max_passwd_query_count;
-    pthread_mutex_unlock(&m_smb_conn);
-    DPRINTF(7, "count=%d\n", count);
-    return count;
+static inline int smb_conn_get_max_passwd_query_count(void){
+    return g_atomic_int_get(&smb_conn_max_passwd_query_count);
 }
 
 int smb_conn_set_server_reply_timeout(int timeout){
     if (timeout < 10) return 0;
     DPRINTF(7, "timeout=%d\n", timeout);
-    pthread_mutex_lock(&m_smb_conn);
-    smb_conn_server_reply_timeout = timeout;
-    pthread_mutex_unlock(&m_smb_conn);
+    g_atomic_int_set(&smb_conn_server_reply_timeout, timeout);
     return 1;
 }
 
-int smb_conn_get_server_reply_timeout(void){
-    int timeout;
-
-    pthread_mutex_lock(&m_smb_conn);
-    timeout = smb_conn_server_reply_timeout;
-    pthread_mutex_unlock(&m_smb_conn);
-    DPRINTF(7, "timeout=%d\n", timeout);
-    return timeout;
+static inline int smb_conn_get_server_reply_timeout(void){
+    return g_atomic_int_get(&smb_conn_server_reply_timeout);
 }
 
-void smb_conn_connection_close(struct smb_conn_ctx *ctx){
+static void smb_conn_connection_close(struct smb_conn_ctx *ctx){
     LIST			*elem;
     struct smb_conn_file	*conn_file;
 
@@ -107,7 +130,7 @@ void smb_conn_connection_close(struct smb_conn_ctx *ctx){
     }
 }
 
-int smb_conn_up_if_broken(struct smb_conn_ctx *ctx){
+static int smb_conn_up_if_broken(struct smb_conn_ctx *ctx){
     if (ctx->conn_fd != -1){
 	if (process_is_smb_conn_alive(ctx->conn_fd)) return 0;
 	smb_conn_connection_close(ctx);
@@ -151,7 +174,7 @@ int smb_conn_ctx_destroy(struct smb_conn_ctx *ctx){
     return result;
 }
 
-int smb_conn_send_password_base(struct smb_conn_ctx *ctx, const char *domain,
+static int smb_conn_send_password_base(struct smb_conn_ctx *ctx, const char *domain,
 				const char *user, const char *password){
     ssize_t			bytes;
     struct iovec		iov[5];
@@ -188,7 +211,7 @@ int smb_conn_send_password_base(struct smb_conn_ctx *ctx, const char *domain,
     return (bytes == (ssize_t) header.query_len) ? 0 : -1;
 }
 
-int smb_conn_send_password(struct smb_conn_ctx *ctx,
+static int smb_conn_send_password(struct smb_conn_ctx *ctx,
 			const char *server, const char *share){
 
 #ifdef HAVE_GNOME_KEYRING
@@ -264,11 +287,11 @@ int smb_conn_send_password(struct smb_conn_ctx *ctx,
 #endif /* HAVE_GNOME_KEYRING */
 }
 
-int smb_conn_process_query_lowlevel_va(
+static int smb_conn_process_query_lowlevel_va(
 			struct smb_conn_ctx *ctx,
 			enum smb_conn_cmd query_cmd,
 			void *query, size_t query_len,
-			int *errno_value,
+			struct smb_conn_query_result *result,
 			void *reply, size_t reply_len,
 			va_list ap){
 
@@ -279,9 +302,12 @@ int smb_conn_process_query_lowlevel_va(
 
     if ((ctx == NULL) || (ctx->conn_fd == -1) ||
 	((query == NULL) || (query_len == 0)) ||
-	(errno_value == NULL) ||
+	(result == NULL) ||
 	((reply == NULL) && (reply_len != 0))) return EINVAL;
 
+    result->process_state = SMB_CONN_PROCESS_STATE_UNKNOWN;
+    result->errno_value = EINVAL;
+
     iov_cnt = 2;
     query_header.query_cmd = query_cmd;
     query_header.debug_level = common_get_smbnetfs_debug_level();
@@ -365,11 +391,13 @@ int smb_conn_process_query_lowlevel_va(
 	    msg = ((char *) msg_req) + msg_req->msg_offs;
 	    if (bytes != (ssize_t) (strlen(msg) + 1)) goto error;
 
-	    common_debug_print(msg_req->debug_level, "%s", msg);
+	    DEBUG_PRINT(msg_req->debug_level, "%s", msg);
 
 	    if (reply_hdr->reply_cmd == DIE_MSG){
-		errno = reply_hdr->errno_value;
-		goto error;
+		result->errno_value   = reply_hdr->errno_value;
+		result->process_state = SMB_CONN_PROCESS_STATE_DIED;
+		smb_conn_connection_close(ctx);
+		return 0;
 	    }
 	    continue;
 	}
@@ -412,7 +440,8 @@ int smb_conn_process_query_lowlevel_va(
 
 	/* ok, we got a reply, store error/reply and exit */
 	if (reply_len != 0) memcpy(reply, reply_hdr + 1, reply_len);
-	*errno_value = reply_hdr->errno_value;
+	result->errno_value   = reply_hdr->errno_value;
+	result->process_state = SMB_CONN_PROCESS_STATE_ALIVE;
 	return 0;
     }
 
@@ -421,11 +450,11 @@ int smb_conn_process_query_lowlevel_va(
     return EIO;
 }
 
-int smb_conn_process_query_lowlevel(
+static int smb_conn_process_query_lowlevel(
 			struct smb_conn_ctx *ctx,
 			enum smb_conn_cmd query_cmd,
 			void *query, size_t query_len,
-			int *errno_value,
+			struct smb_conn_query_result *result,
 			void *reply, size_t reply_len,
 			...){
     va_list	ap;
@@ -436,24 +465,40 @@ int smb_conn_process_query_lowlevel(
 			ctx,
 			query_cmd,
 			query, query_len,
-			errno_value,
+			result,
 			reply, reply_len,
 			ap);
     va_end(ap);
     return retval;
 }
 
-int smb_conn_process_query(
+static int smb_conn_process_query(
 			struct smb_conn_ctx *ctx,
 			enum smb_conn_cmd query_cmd,
 			void *query, size_t query_len,
 			void *reply, size_t reply_len,
 			...){
 
-    va_list		ap;
-    int			count, retval, errno_value;
+    va_list				ap;
+    int					count, retval;
+    struct smb_conn_query_result	result;
+    const char				*url = NULL;
 
-    for(count = 0; count < smb_conn_get_max_retry_count(); count++){
+    if ((query_cmd >= OPEN) && (query_cmd <= REMOVEXATTR)){
+	va_start(ap, reply_len);
+	url = va_arg(ap, const char *);
+	va_end(ap);
+
+	if (url != NULL){
+	    retval = neg_cache_check(url);
+	    if (retval != 0){
+		errno = retval;
+		return retval;
+	    }
+	}
+    }
+
+    for(count = 0; ; count++){
 	if (smb_conn_up_if_broken(ctx) != 0) break;
 
 	va_start(ap, reply_len);
@@ -461,18 +506,25 @@ int smb_conn_process_query(
 			ctx,
 			query_cmd,
 			query, query_len,
-			&errno_value,
+			&result,
 			reply, reply_len,
 			ap);
 	va_end(ap);
+	if ((retval == 0) && smb_conn_query_result_check(query_cmd, &result)){
+	    retval = smb_conn_query_result_map(&result);
+	    if (smb_conn_is_neg_cache_candidate(query_cmd, &result) && (url != NULL))
+		neg_cache_store(url, retval);
+	    return retval;
+	}
 
-	if (retval == 0) return errno_value;
+	if (count >= smb_conn_get_max_retry_count()) break;
 	sleep(2);
     }
+    if (url != NULL) neg_cache_store(url, EIO);
     return EIO;
 }
 
-int smb_conn_process_fd_query(
+static int smb_conn_process_fd_query(
 			struct smb_conn_ctx *ctx,
 			enum smb_conn_cmd query_cmd,
 			struct smb_conn_file *file,
@@ -480,11 +532,12 @@ int smb_conn_process_fd_query(
 			void *query, size_t query_len,
 			void *reply, size_t reply_len){
 
-    int			count, retval, errno_value;
+    int					count, retval;
+    struct smb_conn_query_result	result;
 
     if ((file == NULL) || (file->url == NULL)) return EINVAL;
 
-    for(count = 0; count < smb_conn_get_max_retry_count(); count++){
+    for(count = 0; ; count++){
 	if (smb_conn_up_if_broken(ctx) != 0) break;
 
 	if (file->srv_fd == NULL){
@@ -521,11 +574,12 @@ int smb_conn_process_fd_query(
 			ctx,
 			file->reopen_cmd,
 			&fd_query, fd_len,
-			&errno_value,
+			&result,
 			&fd_reply, sizeof(fd_reply),
 			file->url, NULL);
-	    if (retval != 0) goto loop_end;
-	    if (errno_value != 0) return errno_value;
+	    if ((retval != 0) || !smb_conn_query_result_check(file->reopen_cmd, &result)) goto loop_end;
+	    if (smb_conn_query_result_map(&result) != 0)
+		return smb_conn_query_result_map(&result);
 	
 	    file->srv_fd = fd_reply.srv_fd;
 	}
@@ -535,12 +589,14 @@ int smb_conn_process_fd_query(
 			ctx,
 			query_cmd,
 			query, query_len,
-			&errno_value,
+			&result,
 			reply, reply_len,
 			NULL);
-	if (retval == 0) return errno_value;
+	if ((retval == 0) && smb_conn_query_result_check(query_cmd, &result))
+	    return smb_conn_query_result_map(&result);
 
       loop_end:
+	if (count >= smb_conn_get_max_retry_count()) break;
 	sleep(2);
     }
     return EIO;
@@ -740,15 +796,15 @@ int smb_conn_close(struct smb_conn_ctx *ctx, smb_conn_fd fd){
     if ((file->reopen_cmd == OPEN) && (file->ctx == ctx)){
 	ctx->access_time = time(NULL);
 	if (file->srv_fd != NULL){
-	    int				errno_value;
-	    struct smb_conn_fd_query	query;
+	    struct smb_conn_query_result	result;
+	    struct smb_conn_fd_query		query;
 
 	    query.srv_fd = file->srv_fd;
 	    smb_conn_process_query_lowlevel(
 			ctx,
 			CLOSE,
 			&query, sizeof(query),
-			&errno_value,
+			&result,
 			NULL, 0,
 			NULL);
 	}
@@ -864,15 +920,15 @@ int smb_conn_closedir(struct smb_conn_ctx *ctx, smb_conn_fd fd){
     if ((file->reopen_cmd == OPENDIR) && (file->ctx == ctx)){
 	ctx->access_time = time(NULL);
 	if (file->srv_fd != NULL){
-	    int				errno_value;
-	    struct smb_conn_fd_query	query;
+	    struct smb_conn_query_result	result;
+	    struct smb_conn_fd_query		query;
 
 	    query.srv_fd = file->srv_fd;
 	    smb_conn_process_query_lowlevel(
 			ctx,
 			CLOSEDIR,
 			&query, sizeof(query),
-			&errno_value,
+			&result,
 			NULL, 0,
 			NULL);
 	}
@@ -931,23 +987,22 @@ ssize_t smb_conn_readdir(struct smb_conn_ctx *ctx,
 	    /* connection recovery is not possible */
 	    /* =================================== */
 
-	    int	errno_value = 0;
+	    struct smb_conn_query_result	result;
 
 	    if (file->srv_fd == NULL){
 		errno = EIO;
 		return -1;
 	    }
 
-	    errno_value = 0;
 	    query.srv_fd  = file->srv_fd;
 	    error = smb_conn_process_query_lowlevel(
 			ctx,
 			READDIR,
 			&query, sizeof(query),
-			&errno_value,
+			&result,
 			&reply, sizeof(reply),
 			NULL);
-	    if (error == 0) error = errno_value;
+	    if (error == 0) error = smb_conn_query_result_map(&result);
 	}
 	if ((error == 0) && (reply.bufsize <= (ssize_t) bufsize) &&
 	    (reply.bufsize % sizeof(struct smb_conn_dirent_rec) == 0)){
