@@ -35,13 +35,14 @@
 	((ctx)->callbacks.auth_fn = (ctx_auth_fn##_old))
     #define	smbc_ftruncate(a, b)					\
 	( errno = EINVAL, -1)
+    #define	smbc_setTimeout(ctx, timeout)	do {} while (0)
 #endif
 
 
 void smb_conn_srv_debug_print(struct smb_conn_srv_ctx *ctx,
 				enum smb_conn_cmd msg_type,
 				int errno_value,
-				int level,
+				int level, int no_fallback,
 				const char *fmt, ...){
 
     static char			buf[COMM_BUF_SIZE];
@@ -65,14 +66,14 @@ void smb_conn_srv_debug_print(struct smb_conn_srv_ctx *ctx,
 			errno_value, level, buf) == 0) return;
 
   fallback:
+    if (no_fallback) return;
     va_start(ap, fmt);
     vfprintf(stderr, fmt, ap);
     fflush(stderr);
     va_end(ap);
-    return;
 }
 
-void smb_conn_srv_auth_fn(SMBCCTX *ctx,
+static void smb_conn_srv_auth_fn(SMBCCTX *ctx,
 		const char	*server,
 		const char	*share,
 		char		*wrkgrp, int wrkgrplen,
@@ -185,7 +186,7 @@ void smb_conn_srv_auth_fn(SMBCCTX *ctx,
 }
 
 #ifndef HAVE_LIBSMBCLIENT_3_2
-void smb_conn_srv_auth_fn_old(
+static void smb_conn_srv_auth_fn_old(
 		const char	*server,
 		const char	*share,
 		char		*wrkgrp, int wrkgrplen,
@@ -200,10 +201,11 @@ void smb_conn_srv_auth_fn_old(
 }
 #endif
 
-void smb_conn_srv_samba_init(struct smb_conn_srv_ctx *srv_ctx){
+static void smb_conn_srv_samba_init(struct smb_conn_srv_ctx *srv_ctx){
     SMBCCTX	*ctx;
 
     if ((ctx = smbc_new_context()) == NULL) goto error;
+    smbc_setTimeout(ctx, srv_ctx->smb_timeout);
     smbc_setDebug(ctx, srv_ctx->smb_debug_level);
     smbc_setFunctionAuthDataWithContext(ctx, smb_conn_srv_auth_fn);
     smbc_setOptionUserData(ctx, srv_ctx);
@@ -272,8 +274,8 @@ void smb_conn_srv_listen(struct smb_conn_srv_ctx *ctx){
 	errno = 0;
 	query = (void*) (query_hdr + 1);
 	query_len -= sizeof(struct smb_conn_query_hdr);
-	DSRVPRINTF(ctx, 6, "process query=%d, query_len=%d\n",
-			query_hdr->query_cmd, (int) query_len);
+	DSRVPRINTF(ctx, 6, "process query=%d, query_len=%zd\n",
+			query_hdr->query_cmd, query_len);
 	switch(query_hdr->query_cmd){
 	    case OPEN:
 		smb_conn_srv_open(ctx, query, query_len);
